/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2014 ILS Technology, LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <tr50/error.h>
#include <tr50/util/memory.h>
#include <tr50/util/thread.h>
#if defined(DW_AGENT_FOR_ARM)
#include <tx_api.h>
#else
#include "tal_inc/tal_portab.h"
#include "tal_inc/tal_os.h"

#include <string.h>
#endif

#if defined(DW_AGENT_FOR_ARM)
#define DW_CHILDREN_THREADS_STACK_SIZE 4096
#define DW_CHILDREN_THREADS_PRIO       128
#else
#define DW_CHILDREN_THREADS_STACK_SIZE 32768
#define DW_CHILDREN_THREADS_PRIO       105
#endif

typedef struct {
  char name[32];
  void *stack;
#if defined(DW_AGENT_FOR_ARM)
  TX_THREAD tx;
#else
  TalOsThread thr;
#endif
  _thread_function func;
  void *arg;
} _TELIT_THREAD;

void _thread_handler (void *handle) {
	_TELIT_THREAD *thread=(_TELIT_THREAD*)handle;
	thread->func(thread->arg);
}

#if !defined(DW_AGENT_FOR_ARM)
static void _thread_delete_cb(TalOsThread* thread)
{

}
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int _thread_create(void **handle, const char* name, _thread_function func, void *arg) {
	_TELIT_THREAD *thread=(_TELIT_THREAD*)_memory_malloc(sizeof(_TELIT_THREAD));
	_memory_memset(thread,0,sizeof(_TELIT_THREAD));
	
	thread->stack=(void*)_memory_malloc(DW_CHILDREN_THREADS_STACK_SIZE);
	_memory_memset(thread->stack,0,DW_CHILDREN_THREADS_STACK_SIZE);

	thread->func=func;
	thread->arg=arg;

	strcpy(thread->name,name);
	/* Create the task */
#if defined(DW_AGENT_FOR_ARM)
	if ( tx_thread_create( &thread->tx, thread->name,
                           (VOID (*)(ULONG))_thread_handler,(ULONG) thread, 
                           thread->stack, DW_CHILDREN_THREADS_STACK_SIZE, 
                           DW_CHILDREN_THREADS_PRIO, DW_CHILDREN_THREADS_PRIO,
                           2, TX_AUTO_START ) != TX_SUCCESS )
#else
	if ( TalOsThreadCreate(&thread->thr,
                          (TalChar *)thread->name, (TalOsThreadPriority)DW_CHILDREN_THREADS_PRIO, 
                          thread->stack, (TalUInt32)DW_CHILDREN_THREADS_STACK_SIZE,
                          (TalOsThreadFn)_thread_handler,
                          thread) != TAL_SUCCESS )
#endif
  {
    //log_should_not_happen("tx_thread_create(): failed.");
    return -1;
  }

#if !defined(DW_AGENT_FOR_ARM) && defined(TAL_INTERFACE)
  TalOsThreadRegisterDeletedHandler(&thread->thr, _thread_delete_cb);

  TalOsThreadStart(&thread->thr);
#endif

  *handle=thread;
  return 0;                           
}

int _thread_join(void *handle) {
	_thread_sleep(5000);
	return 0;
}

int _thread_delete(void *handle) {
  _TELIT_THREAD *thread=(_TELIT_THREAD*)handle;

#if defined(DW_AGENT_FOR_ARM)
  tx_thread_terminate( &thread->tx );
  tx_thread_delete( &thread->tx );
#else
  TalOsThreadDelete(&thread->thr);
#endif
    
  _memory_free( thread->stack );
  _memory_free( thread );
	return 0;
}

void _thread_sleep(int ms) {
#if defined(DW_AGENT_FOR_ARM)
  tx_thread_sleep(ms/10);
#else
  TalOsSleep(ms,ms);
#endif
}

int _thread_id(int *id) {
	return ERR_TR50_NOPORT;
}

